SystemOrganization addCategory: #'Dominion-Cards'!
SystemOrganization addCategory: #Dominion!
SystemOrganization addCategory: #'Dominion-Test'!

OrderedCollection subclass: #CardArea
	instanceVariableNames: 'visibility'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!CardArea class methodsFor: 'instance creation' stamp: 'JohnParker 7/17/2012 21:15'!
visibility: aSymbol
	^self new visibility: aSymbol.! !

!CardArea methodsFor: 'accessing' stamp: 'JohnParker 7/17/2012 21:16'!
visibility 
	^visibility.! !

!CardArea methodsFor: 'initialize-release' stamp: 'JohnParker 7/17/2012 21:45'!
visibility: symbol1
	visibility := symbol1.! !

TestCase subclass: #CoreTest
	instanceVariableNames: 'board deck1 deck2 player1 player2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Test'!

!CoreTest methodsFor: 'private' stamp: 'JohnParker 7/18/2012 12:12'!
setupGame
	board := Board new.
	board initialize.
	board decks: 2.
	deck1 := board deck: 1.
	deck2 := board deck: 2.

	player1 := TestPlayer new.
	board addObserver: player1 at: 1.
	board addObserver: player1.
	player1 board: board deck: 1.
	player2 := TestPlayer new.
	board addObserver: player2 at: 2.
	board addObserver: player2.
	player2 board: board deck: 2.
	
	board startGame.! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/17/2012 20:34'!
testActionCommand
	self setupGame.
	
	"Send an action command to the wrong deck.  Make sure it returns an error."
	board execute: (ActionCommand deck: 2 card: Smithy).
	self assert: player2 errorCommand isNil not.
	self assert: player1 errorCommand isNil.
	player2 clearError.
	
	"Send an action command on a non action card.  Make sure it returns an error."
	board execute: (ActionCommand deck: 1 card: Copper).
	self assert: player1 errorCommand isNil not.
	self assert: player2 errorCommand isNil.
	player1 clearError.
	
	"Send an ActionCommand to deck1 with a smithy.  Make sure that the card is played to the play area, 3 more cards are drawn by the smithy, and the phase is advanced to Treasure"
	deck1 z_addCardToHand: Smithy.
	board execute: (ActionCommand deck: 1 card: Smithy).
	self assert: player1 errorCommand isNil.
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 8.
	self assert: (deck1 play at: 1) class = Smithy.
	self assert: deck1 phase = #Treasure.
	
	"Send an action command after the action phase is complete.  Make sure it returns an error."
	deck1 z_addCardToHand: Smithy.
	board execute: (ActionCommand deck: 1 card: Smithy).
	self assert: player1 errorCommand isNil not.
	self assert: player2 errorCommand isNil.
	player1 clearError.
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/17/2012 20:01'!
testAdventurer
	|treasure|
	self setupGame.
	
	self assert: Adventurer cost = 6.
	
	"Send an ActionCommand to deck1 with a smithy.  Make sure that the card is played to the play area, 3 more cards are drawn by the smithy, and the phase is advanced to Treasure"
	treasure := (deck1 hand select: [:each | each class inheritsFrom: TreasureCard]) size.
	deck1 z_addCardToHand: Adventurer.
	board execute: (ActionCommand deck: 1 card: Adventurer).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 7.
	self assert: (deck1 hand select: [:each | each class inheritsFrom: TreasureCard]) size = (treasure + 2).
	self assert: (deck1 play at: 1) class = Adventurer.
	self assert: deck1 phase = #Treasure.
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/17/2012 20:02'!
testBureaucrat
	self setupGame.
	
	self assert: Bureaucrat cost = 4.
	
	"Play a Bureaucrat.  Make sure that a silver is added to the top of the draw and that other players either placed a victory card on top of the draw or have no victory cards."
	deck1 z_addCardToHand: Bureaucrat.
	board execute: (ActionCommand deck: 1 card: Bureaucrat).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 5.
	self assert: deck1 z_draw last class = Silver.
	self assert: (((deck2 hand size = 4) and: (deck2 z_draw last class = Estate)) or: ((deck2 hand select: [:each | each class = Copper]) size = 5)).
	self assert: (deck1 play at: 1) class = Bureaucrat.
	self assert: deck1 phase = #Treasure.
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/17/2012 20:34'!
testBuyCommand
	self setupGame.
	board addCard: Adventurer num: 10.
	
	"Make sure the phase of each deck is correct.  Deck 1 should be at Action since that is the first phase of a turn.  Deck 2 should be at None because it is not Deck 2s turn."
	self assert: deck1 phase = #Action.
	self assert: deck2 phase = #None.
	
	"Send a buy command to the correct deck.  Make sure it successfully adds the card to the discard and ends the turn"
	board execute: (BuyCommand deck: 1 card: Copper).
	self assert: player1 errorCommand isNil.
	self assert: deck1 z_discard size = 6.
	self assert: (deck1 z_discard at: 1) class = Copper.
	self assert: deck1 phase = #None.
	self assert: deck2 = board currentDeck.
	
	"Send a buy command to the wrong deck.  Make sure it returns an error."
	board execute: (BuyCommand deck: 1 card: Copper).
	self assert: player1 errorCommand isNil not.
	self assert: player2 errorCommand isNil.
	player1 clearError.
	
	"Send a buy command to the new correct deck.  Make sure it successfully adds the card to the discard and ends the turn"
	board execute: (BuyCommand deck: 2 card: Copper).
	self assert: player2 errorCommand isNil.
	self assert: deck2 z_discard size = 6.
	self assert: (deck1 z_discard at: 1) class = Copper.
	self assert: deck2 phase = #None.
	self assert: deck1 = board currentDeck.
	
	board execute: (BuyCommand deck: 1 card: Adventurer).
	self assert: player1 errorCommand isNil not.
	self assert: player2 errorCommand isNil.
	player1 clearError.
	self assert: deck1 z_discard size = 6.
	self assert: (deck1 z_discard at: 1) class = Copper.
	self assert: deck1 phase = #Action.
	self assert: deck1 = board currentDeck.! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/17/2012 20:02'!
testCellar
	| victory | 
	self setupGame.
	
	self assert: Cellar cost = 2.
	
	"Play a Cellar.  We are using a default player who will discard all of his victory cards, so make sure he does so and that the correct number of cards are drawn to replace them."
	victory := (deck1 hand select: [:each | each class inheritsFrom: VictoryCard]) size.
	deck1 z_addCardToHand: Cellar.
	board execute: (ActionCommand deck: 1 card: Cellar).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 5.
	self assert: (deck1 z_discard size = victory).
	self assert: ((1 to: victory) select: [:n | (deck1 z_discard at: n) class = Estate]) size = victory.
	self assert: (deck1 play at: 1) class = Cellar.
	self assert: deck1 phase = #Action.
	self assert: deck1 actions = 1.
	self assert: deck1 z_resolving = false.
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/17/2012 20:35'!
testEndTurnCommand
	self setupGame.
	
	"Send an end turn command to the wrong deck.  Make sure it returns an error."
	board execute: (EndTurnCommand deck: 2).
	self assert: player2 errorCommand isNil not.
	self assert: player1 errorCommand isNil.
	player2 clearError.
	
	"Send an end turn command to deck1."
	"Before: draw(5) discard(0) hand(5) play(0)"
	"After:    draw(0) discard(5) hand(5) play(0)"
	board execute: (EndTurnCommand deck: 1).
	self assert: player2 errorCommand isNil.
	self assert: deck1 play size = 0.
	self assert: deck1 hand size = 5.
	self assert: deck1 z_discard size = 5.
	self assert: deck1 z_draw size = 0.
	self assert: deck1 phase = #None.
	
	"Send an end turn command to the wrong deck.  Make sure it returns an error."
	board execute: (EndTurnCommand deck: 1).
	self assert: player1 errorCommand isNil not.
	self assert: player2 errorCommand isNil.
	player1 clearError.
	
	"Send an end turn command to deck2."
	"Before: draw(5) discard(0) hand(5) play(0)"
	"After:    draw(0) discard(5) hand(5) play(0)"
	board execute: (EndTurnCommand deck: 2).
	self assert: player2 errorCommand isNil.
	self assert: deck2 play size = 0.
	self assert: deck2 hand size = 5.
	self assert: deck2 z_discard size = 5.
	self assert: deck2 z_draw size = 0.
	self assert: deck2 phase = #None.
	
	"Send an end turn command to the wrong deck.  Make sure it returns an error."
	board execute: (EndTurnCommand deck: 2).
	self assert: player2 errorCommand isNil not.
	self assert: player1 errorCommand isNil.
	player2 clearError.
	
	"Send a play treasure command to deck1."
	"Before: draw(0) discard(5) hand(5) play(0)"
	"After:    draw(0) discard(5) hand(4) play(1)"
	board execute: (TreasureCommand deck: 1 card: Copper).
	self assert: player1 errorCommand isNil.
	
	"Send a buy command to deck1.  This also ends the turn because the deck is out of buys"
	"Before:     draw(0) discard(5) hand(4) play(1)"
	"After buy: draw(0) discard(6) hand(4) play(1)"
	"After end: draw(6) discard(0) hand(5) play(0)"
	board execute: (BuyCommand deck: 1 card: Copper).
	self assert: player1 errorCommand isNil.
	self assert: deck1 play size = 0.
	self assert: deck1 hand size = 5.
	self assert: deck1 z_discard size = 0.
	self assert: deck1 z_draw size = 6.
	self assert: deck1 phase = #None.
	
	"Send an end turn command to the wrong deck.  Make sure it returns an error."
	board execute: (EndTurnCommand deck: 1).
	self assert: player1 errorCommand isNil not.
	self assert: player2 errorCommand isNil.
	player1 clearError.
	
	"Send an end turn command to deck2."
	"Before: draw(0) discard(5) hand(5) play(0)"
	"After:   draw(5) discard(0) hand(5) play(0)"
	board execute: (EndTurnCommand deck: 2).
	self assert: player2 errorCommand isNil.
	self assert: deck2 play size = 0.
	self assert: deck2 hand size = 5.
	self assert: deck2 z_discard size = 0.
	self assert: deck2 z_draw size = 5.
	self assert: deck2 phase = #None.! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/17/2012 20:04'!
testGameEnd
	self testSetup.
	
	1 to: 7 do: [:a |
		1 to: 3 do: [:b | deck1 z_addCardToHand: Gold].
		1 to: 3 do: [:b |board execute: (TreasureCommand deck: 1 card: Gold)].
		board execute: (BuyCommand deck: 1 card: Province).
		
		board execute: (EndTurnCommand deck: 2).
		
		self assert: board gameOver not.
		self should: [deck1 victoryPoints] raise: Error.
	].
	
	1 to: 3 do: [:b | deck1 z_addCardToHand: Gold].
	1 to: 3 do: [:b |board execute: (TreasureCommand deck: 1 card: Gold)].
	board execute: (BuyCommand deck: 1 card: Province).
	
	self assert: board gameOver.
	self assert: board winner = 1.
	self assert: deck1 victoryPoints = ((6*8)+3).
	self assert: deck2 victoryPoints = 3.! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/18/2012 12:09'!
testMine
	| copper | 
	self setupGame.
	
	self assert: Mine cost = 5.
	
	"Play a Mine.  We are using a default player who will trash copper for silver, so make sure he does so."
	copper := (deck1 hand select: [:each | each class = Copper]) size.
	deck1 z_addCardToHand: Mine.
	board execute: (ActionCommand deck: 1 card: Mine).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 5.
	self assert: board trashTop class = Copper.
	self assert: (deck1 z_discard size = 0).
	self assert: (deck1 hand select: [:each | each class = Copper]) size = (copper - 1).
	self assert: (deck1 handContains: Silver).
	self assert: (deck1 play at: 1) class = Mine.
	self assert: deck1 phase = #Treasure.
	self assert: deck1 z_resolving = false.
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/18/2012 20:14'!
testMoat
	self setupGame.
	
	self assert: Moat cost = 2.
	
	"Play a Bureaucrat.  Make sure that nothing happens because player 2 reveals the moat."
	deck1 z_addCardToHand: Bureaucrat.
	deck2 z_addCardToHand: Moat.
	deck2 z_addCardToHand: Estate.
	board execute: (ActionCommand deck: 1 card: Bureaucrat).
	self assert: deck2 hand size = 7.
	
	self setupGame.
	
	"Play a Moat. Make sure 2 cards are drawn"
	deck1 z_addCardToHand: Moat.	

	board execute: (ActionCommand deck: 1 card: Moat).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 7.
	self assert: deck1 actions = 0.
	self assert: (deck1 play at: 1) class = Moat.
	self assert: deck1 phase = #Treasure.

	
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/17/2012 13:05'!
testSetup
	| actions cards temp|
	self setupGame.

	self assert: (deck1 z_draw select: [:each | each class = Copper]) size + (deck1 hand select: [:each | each class = Copper]) size= 7.
	self assert: (deck1 z_draw select: [:each | each class = Estate]) size + (deck1 hand select: [:each | each class = Estate]) size= 3.
	
	self assert: (deck2 z_draw select: [:each | each class = Copper]) size + (deck2 hand select: [:each | each class = Copper]) size= 7.
	self assert: (deck2 z_draw select: [:each | each class = Estate]) size + (deck2 hand select: [:each | each class = Estate]) size= 3.
	
	self assert: ((board cardsRemaining: Copper) = (60 - (7*2))).
	self assert: ((board cardsRemaining: Estate) = 8).
	
	actions := {Adventurer. Village. Smithy. Cellar. Mine.}.
	cards := {Copper. Silver. Gold. Estate. Duchy. Province. Curse. Adventurer. Village. Smithy. Cellar. Mine.}.
	
	actions do: [:each | board addCard: each num: 10].
	
	temp := board cardsAvailable.
	self assert: (temp includesAll: cards).
	self assert: (cards includesAll: temp).
	
	temp := board actionsAvailable.
	self assert: (temp includesAll: actions).
	self assert: (actions includesAll: temp).
	actions do: [:each | board cardsRemaining: each = 10].
	
	self assert: (board cardStackEmpty: Village) not.
	10 timesRepeat: [board takeCard: Village deck: 1].
	self assert: (board cardStackEmpty: Village).
	
! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/17/2012 20:04'!
testSmithy
	self setupGame.
	
	self assert: Smithy cost = 4.
	
	"Play a Smithy. Make sure 3 cards are drawn"

	deck1 z_addCardToHand: Smithy.	

	board execute: (ActionCommand deck: 1 card: Smithy).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 8.
	self assert: deck1 actions = 0.
	self assert: (deck1 play at: 1) class = Smithy.
	self assert: deck1 phase = #Treasure.

	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/18/2012 22:00'!
testSpy
	self setupGame.
	
	self assert: Spy cost = 4.
	
	"Play a Spy.  Make sure that the player draws 2 cards and other players gain a curse."
	deck1 z_draw addLast: Estate basicNew.
	deck2 z_draw addLast: Copper basicNew.
	deck1 z_addCardToHand: Spy.
	board execute: (ActionCommand deck: 1 card: Spy).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 6.
	self assert: deck1 actions = 1.
	self assert: (deck1 play at: 1) class = Spy.
	self assert: deck1 phase = #Action.
	self assert: deck1 z_resolving = false.
	self assert: deck1 z_draw size = 4.
	self assert: deck2 z_draw size = 4.
	self assert: deck1 z_discard last = Estate.
	self assert: deck2 z_discard last = Copper.
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/17/2012 20:37'!
testTreasureCommand
	self setupGame.
	
	"Send a TreasureCommand to deck1.  Make sure that the card is played to the play area and that the money is increased as expected."
	board execute: (TreasureCommand deck: 1 card: Copper).
	self assert: player1 errorCommand isNil.
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 4.
	self assert: deck1 money = 1.
	self assert: (deck1 play at: 1) class = Copper.
	self assert: deck1 phase = #Treasure.
	
	"Send a treasure command to the wrong deck.  Make sure it returns an error."
	board execute: (TreasureCommand deck: 2 card: Copper).
	self assert: player2 errorCommand isNil not.
	self assert: player1 errorCommand isNil.
	player2 clearError.
	
	"Send a TreasureCommand to deck1.  Make sure that the card is played to the play area and that the money is increased as expected."
	board execute: (TreasureCommand deck: 1 card: Copper).
	self assert: player1 errorCommand isNil.
	self assert: deck1 play size = 2.
	self assert: deck1 hand size = 3.
	self assert: deck1 money = 2.
	self assert: (deck1 play at: 1) class = Copper.
	self assert: deck1 phase = #Treasure.
	
	"Try to buy an estate.  This costs 2, which we have due to having played 2 coppers"
	board execute: (BuyCommand deck: 1 card: Estate).
	self assert: player1 errorCommand isNil.
	self assert: deck1 z_discard size = 6.
	self assert: (deck1 z_discard at: 1) class = Estate.
	self assert: deck1 phase = #None.
	self assert: deck2 = board currentDeck.! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/17/2012 20:05'!
testVillage
	self setupGame.
	
	self assert: Village cost = 3.
	
	"Play a Village. Make sure a card was drawn, the number of actions is now 2, and the phase is still #Action"
	deck1 z_addCardToHand: Village.
	deck1 z_addCardToHand: Smithy.
	deck1 z_addCardToHand: Smithy.
	board execute: (ActionCommand deck: 1 card: Village).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 8.
	self assert: deck1 actions = 2.
	self assert: (deck1 play at: 1) class = Village.
	self assert: deck1 phase = #Action.
	
	"Make sure we can now play two more cards."
	board execute: (ActionCommand deck: 1 card: Smithy).
	self assert: deck1 play size = 2.
	self assert: deck1 hand size = 10.
	self assert: deck1 actions = 1.
	self assert: (deck1 play at: 2) class = Smithy.
	self assert: deck1 phase = #Action.
	
	board execute: (ActionCommand deck: 1 card: Smithy).
	self assert: deck1 play size = 3.
	self assert: deck1 hand size = 10.
	self assert: deck1 actions = 0.
	self assert: (deck1 play at: 3) class = Smithy.
	self assert: deck1 phase = #Treasure.
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/18/2012 20:26'!
testWitch
	self setupGame.
	
	self assert: Witch cost = 5.
	
	"Play a Witch.  Make sure that the player draws 2 cards and other players gain a curse."
	deck1 z_addCardToHand: Witch.
	board execute: (ActionCommand deck: 1 card: Witch).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 7.
	self assert: (deck1 play at: 1) class = Witch.
	self assert: deck1 phase = #Treasure.
	self assert: (deck2 z_discard last class = Curse).
	
	! !

Object subclass: #Board
	instanceVariableNames: 'cards decks trash currentDeck gameOver globalEvents playerEvents'
	classVariableNames: 'UniqueInstance'
	poolDictionaries: ''
	category: 'Dominion'!

!Board commentStamp: 'JohnParker 7/17/2012 22:04' prior: 0!
A Board is a singleton representing the overall game state of Dominion.

Instance Variables
	cards:			Bag
	currentDeck:	Number
	decks:			OrderedCollection
	trash: 			OrderedCollection 
	gameOver:		Boolean
	globalEvents	Array
	playerEvents	Array

cards
	- a Bag containing the card classes available and the number left of each class

currentDeck
	- the deck whose turn it currently is.

decks
	- the list of decks in the game.  

trash
	- a list of cards that have been trashed so far.

gameOver 
	- indicates whether the game is over.

globalEvents
	- the events that are sent to all players when triggered.
	Signatures:
		deck: aNumber cardMovedGlobal: aCardClass from: area1 to: area2
		deck: aNumber took: aCardClass
		deck: aNumber trashed: aCardClass
		turnStarted: aNumber
		deckShuffled: aNumber	
		
playerEvents
	- the events that are sent to a specific player when triggered.  These events carry information that is private to a given player.
	Signatures
		deck: aNumber cardMovedPlayer: aCardClass from: area1 to: area2
		resolveAction: aCardClass
		respondToCard: UNDEFINED
		command: aCommand error: aString!

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/17/2012 12:38'!
actionsAvailable
	^cards asSet select: [:each | each inheritsFrom: ActionCard].! !

!Board methodsFor: 'setup' stamp: 'JohnParker 7/3/2012 20:45'!
addCard: aCardClass num: aNumber
	cards add: aCardClass withOccurrences: aNumber! !

!Board methodsFor: 'setup' stamp: 'JohnParker 7/18/2012 12:08'!
addObserver: anObserver
	globalEvents do: [:event | self when: event send: event to: anObserver].
	! !

!Board methodsFor: 'setup' stamp: 'JohnParker 7/17/2012 12:18'!
addObserver: anObserver at: deckNum
	playerEvents do: [:event | (decks at: deckNum) when: event send: event to: anObserver].
	! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/3/2012 21:38'!
cardStackEmpty: aCardClass
	^(cards includes: aCardClass) not! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/17/2012 12:36'!
cardsAvailable
	^cards asSet! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 02:27'!
cardsRemaining: aCardClass
	^cards occurrencesOf: aCardClass! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 00:56'!
currentDeck
	^decks at: currentDeck! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 00:29'!
deck: aNumber
	^decks at: aNumber ifAbsent: [^nil]! !

!Board methodsFor: 'setup' stamp: 'JohnParker 7/18/2012 11:42'!
decks: aNumber
	| victoryNumber deck|
	cards := Bag new.
	
	"Add the standard cards to the game"
	cards add: Copper withOccurrences: 60.
	cards add: Silver withOccurrences: 40.
	cards add: Gold withOccurrences: 30.
	
	victoryNumber := (aNumber = 2) ifTrue: [8] ifFalse: [12].
	cards add: Estate withOccurrences: (victoryNumber + (3*aNumber)).
	cards add: Duchy withOccurrences: victoryNumber.
	cards add: Province withOccurrences: victoryNumber.
	cards add: Curse withOccurrences: (10 * (aNumber - 1)).
	
	1 to: aNumber do: [:num | 
		deck := Deck number: num board: self.
		decks add: deck.
		globalEvents do: [:event | deck when: event send: event to: self].
	].
	currentDeck := 1.
	! !

!Board methodsFor: 'events' stamp: 'JohnParker 7/13/2012 13:30'!
doesNotUnderstand: aMessage
	(globalEvents includes: aMessage selector) ifFalse: [super doesNotUnderstand: aMessage].
	self triggerEvent: aMessage selector withArguments: aMessage arguments
	! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 21:28'!
events
	|list|
	list := OrderedCollection new.
	list addAll: globalEvents.
	list addAll: playerEvents.
	^list.! !

!Board methodsFor: 'game' stamp: 'JohnParker 7/18/2012 12:01'!
execute: aCommand
	aCommand do: self.! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 16:39'!
gameOver
	^gameOver.! !

!Board methodsFor: 'initialize-release' stamp: 'JohnParker 7/18/2012 21:26'!
initialize
	decks := OrderedCollection new.
	currentDeck := 1.
	cards := Bag new.
	trash := OrderedCollection new.
	gameOver := false.
	globalEvents := {#deck:cardMovedGlobal:from:to:. #turnStarted:. #deckShuffled. #deck:took:. #deck:trashed:.}.
	playerEvents := {#deck:cardMovedPlayer:from:to:. #resolveAction:. #resolveAction:deck:. #attack:. #command:error:.}.! !

!Board methodsFor: 'game' stamp: 'JohnParker 7/5/2012 21:07'!
nextDeck
	(cards includes: Province) ifFalse: [
		gameOver := true.
		currentDeck := 0.
	] ifTrue: [
		currentDeck := (currentDeck rem: decks size) + 1.
		(decks at: currentDeck) startTurn.
	]! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/18/2012 20:39'!
numDecks
	^decks size.! !

!Board methodsFor: 'game' stamp: 'JohnParker 7/4/2012 23:43'!
otherDecks
	^((1 to: decks size) reject: [:num | num = currentDeck]) collect: [:num | decks at: num].! !

!Board methodsFor: 'setup' stamp: 'JohnParker 7/4/2012 13:49'!
startGame
	(decks at: 1) startTurn.! !

!Board methodsFor: 'game' stamp: 'JohnParker 7/13/2012 13:01'!
takeCard: aCardClass deck: aNumber
	cards remove: aCardClass ifAbsent: [^nil].
	self triggerEvent: #deck:took: withArguments: {aNumber. aCardClass}.
	^aCardClass basicNew.! !

!Board methodsFor: 'game' stamp: 'JohnParker 7/13/2012 13:01'!
trashCard: aCard deck: deckNumber
	trash addLast: aCard.
	self triggerEvent: #deck:trashed: withArguments: {deckNumber. aCard class}.! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/17/2012 12:41'!
trashTop
	^trash last.! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 16:39'!
winner
	|tempDecks|
	gameOver ifFalse: [self error: 'The game is not over so there is no winner yet.'].
	tempDecks := OrderedCollection new.
	tempDecks addAll: decks.
	tempDecks sort: [:first :second | first victoryPoints > second victoryPoints].
	^decks indexOf: (tempDecks at: 1).! !

Object subclass: #Card
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!Card commentStamp: 'JohnParker 7/3/2012 19:31' prior: 0!
A Card is an abstract class at the top of the card hierarchy.  It represents a generic card in Dominion.!

Card subclass: #ActionCard
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!ActionCard commentStamp: 'JohnParker 7/4/2012 14:49' prior: 0!
An ActionCard is an abstract class representing a Dominion action card.  !

!ActionCard methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 13:55'!
cardName
	^'Action'! !

!ActionCard methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/10/2012 22:11'!
color
	^Color white! !

!ActionCard methodsFor: 'as yet unclassified' stamp: 'JohnParker 7/18/2012 12:07'!
notifyBadResolve: argArray board: board message: aString 
	board currentDeck triggerEvent: #badCommand withArguments: {#resolveAction. self class. argArray. aString}.
	! !

!ActionCard methodsFor: 'game' stamp: 'JohnParker 7/18/2012 11:57'!
resolve: argVector board: board! !

ActionCard subclass: #Adventurer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Adventurer class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 23:01'!
cardName
	^'Adventurer'! !

!Adventurer class methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 22:04'!
cost
	^6! !

!Adventurer methodsFor: 'game' stamp: 'JohnParker 7/18/2012 21:18'!
playCard: board
	| numTreasure card deck|
	numTreasure := 0.
	deck := board currentDeck.
	[numTreasure < 2] whileTrue: [
		card := deck drawCardTo: #peek.
		card isNil ifTrue: [numTreasure := 2].
		
		(card class inheritsFrom: TreasureCard) ifTrue: [
			deck moveCard: card class from: #peek to: #hand.
			numTreasure := numTreasure + 1
		] ifFalse: [
			deck moveCard: card class from: #peek to: #discard.
		].
	].! !

ActionCard subclass: #AttackCard
	instanceVariableNames: 'playersResolved'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!AttackCard methodsFor: 'game' stamp: 'JohnParker 7/18/2012 12:58'!
resolveAttack: args board: board! !

AttackCard subclass: #Bureaucrat
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Bureaucrat class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 23:30'!
cardName
	^'Bureaucrat'! !

!Bureaucrat class methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 23:30'!
cost
	^4! !

!Bureaucrat methodsFor: 'game' stamp: 'JohnParker 7/18/2012 21:18'!
playCard: board
	playersResolved := OrderedCollection new.
	board numDecks timesRepeat: [playersResolved add: false].
	playersResolved at: board currentDeck number put: true.
	
	board currentDeck gainCard: Silver to: #draw.
	board otherDecks do: [:each | each attack: self].! !

!Bureaucrat methodsFor: 'game' stamp: 'JohnParker 7/18/2012 21:04'!
resolveAttack: args board: board deck: deck
	|cardClass card|
	
	(playersResolved at: deck number) ifTrue: [self error: 'That attack has already been resolved.'].
	
	cardClass := args at: #bureaucratCard ifAbsent: [
		((deck hand select: [:each | each class inheritsFrom: VictoryCard]) size = 0) ifFalse: [
			self error: 'You must choose a victory card if you have one.'
		]
		deck hand do: [:each | deck moveCard: each class from: #hand to: #reveal].
		deck reveal do: [:each | deck moveCard: each class from: #reveal to: #hand].
		playersResolved at: deck number put: true.
		board currentDeck cardResolved: (playersResolved inject: true into: [:val :each | val and: each]).
		^self.
	].
	
	(cardClass inheritsFrom: VictoryCard) ifFalse: [self error: 'The card you choose must be a victory card.'].
	
	card := deck hand detect: [:each | each class = cardClass] ifNone: [self error: 'You must choose a card you have in your hand.'].
	
	deck moveCard: card class from: #hand to: #draw.
	playersResolved at: deck number put: true.
	board currentDeck cardResolved: (playersResolved inject: true into: [:val :each | val and: each]).
	! !

AttackCard subclass: #Spy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Spy class methodsFor: 'accessing' stamp: 'JohnParker 7/18/2012 20:27'!
cardName
	^'Spy'! !

!Spy class methodsFor: 'accessing' stamp: 'JohnParker 7/18/2012 20:27'!
cost
	^4! !

!Spy methodsFor: 'game' stamp: 'JohnParker 7/18/2012 22:07'!
playCard: board
	playersResolved := OrderedCollection new.
	board numDecks timesRepeat: [playersResolved add: false].

	board currentDeck drawCardTo: #hand.
	board currentDeck addActions: 1.
	board otherDecks do: [:each | each attack: self].
	
	board currentDeck drawCardTo: #reveal.
	board currentDeck triggerEvent: #resolveAction:deck: withArguments: {self class}.! !

!Spy methodsFor: 'game' stamp: 'JohnParker 7/18/2012 21:52'!
resolve: args board: board
	| deckNumber discard deck|
	
	deckNumber := args at: #deck.
	discard := args at: #discard.
	
	(playersResolved at: deckNumber) ifTrue: [self error: 'That player has already been resolved.'].
	
	deck := board deck: deckNumber.
	discard ifTrue: [
		deck moveCard: deck reveal last from: #reveal to: #discard
	] ifFalse: [
		deck moveCard: deck reveal last from: #reveal to: #draw
	].
	
	playersResolved at: deckNumber number put: true.
	board currentDeck cardResolved: (playersResolved inject: true into: [:val :each | val and: each]).! !

!Spy methodsFor: 'game' stamp: 'JohnParker 7/18/2012 21:26'!
resolveAttack: args board: board deck: deck
	(playersResolved at: deck number) ifTrue: [self error: 'That attack has already been resolved.'].
	deck drawCardTo: #reveal.
	
	board currentDeck triggerEvent: #resolveAction:deck: withArguments: {self class. deck number}.
	! !

AttackCard subclass: #Witch
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Witch class methodsFor: 'accessing' stamp: 'JohnParker 7/18/2012 20:20'!
cardName
	^'Witch'! !

!Witch class methodsFor: 'accessing' stamp: 'JohnParker 7/18/2012 20:20'!
cost
	^5! !

!Witch methodsFor: 'game' stamp: 'JohnParker 7/18/2012 21:19'!
playCard: board
	playersResolved := OrderedCollection new.
	board numDecks timesRepeat: [playersResolved add: false].
	playersResolved at: board currentDeck number put: true.

	2 timesRepeat: [board currentDeck drawCardTo: #hand ].
	board otherDecks do: [:each | each attack: self].! !

!Witch methodsFor: 'game' stamp: 'JohnParker 7/18/2012 21:04'!
resolveAttack: args board: board deck: deck
	(playersResolved at: deck number) ifTrue: [self error: 'That attack has already been resolved.'].
	deck gainCard: Curse to: #discard.
	
	playersResolved at: deck number put: true.
	board currentDeck cardResolved: (playersResolved inject: true into: [:val :each | val and: each]).
	! !

ActionCard subclass: #Cellar
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Cellar class methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 22:04'!
cardName
	^'Cellar'! !

!Cellar class methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 23:30'!
cost
	^2! !

!Cellar methodsFor: 'game' stamp: 'JohnParker 7/18/2012 21:17'!
playCard: board
	board currentDeck addActions: 1.
	board currentDeck cardResolved: false.
	board currentDeck triggerEvent: #resolveAction: withArguments: {self class}.
	! !

!Cellar methodsFor: 'game' stamp: 'JohnParker 7/18/2012 21:38'!
resolve: args board: board
	| deck discards|
	deck := board currentDeck.
	
	discards := args at: #discards.
	
	(deck handContainsAll: discards) ifFalse: [
		deck triggerEvent: #badCommand withArguments: {#resolveAction. self class. discards.}.
		^self.
	].
	
	discards do: [:each | deck moveCard: each from: #hand to: #discard].
	
	1 to: discards size do: [:a | deck drawCardTo: #hand].
	board currentDeck cardResolved: true.! !

ActionCard subclass: #Mine
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Mine class methodsFor: 'accessing' stamp: 'JohnParker 7/6/2012 01:23'!
cardName
	^'Cellar'! !

!Mine class methodsFor: 'accessing' stamp: 'JohnParker 7/17/2012 12:48'!
cost
	^5! !

!Mine methodsFor: 'game' stamp: 'JohnParker 7/18/2012 21:18'!
playCard: board
	board currentDeck cardResolved: false.
	board currentDeck triggerEvent: #resolveAction: withArguments: {self class}.
	! !

!Mine methodsFor: 'game' stamp: 'JohnParker 7/18/2012 21:37'!
resolve: args board: board
	| deck trashCard gainCard|
	deck := board currentDeck.
	
	trashCard := args at: #trashCard.
	gainCard := args at: #gainCard.
	
	(deck handContains: trashCard) ifFalse: [
		self notifyBadResolve: args board: board message: ('You do not have a ' , trashCard cardName , 'in your hand to trash.').
		^self.
	].
	
	((trashCard inheritsFrom: TreasureCard) and: (gainCard inheritsFrom: TreasureCard)) ifFalse: [
		self notifyBadResolve: args board: board message: ('Mine only works on treasure cards.').
		^self.
	].

	((trashCard cost + 3) >= gainCard cost) ifFalse: [
		self notifyBadResolve: args board: board message: ('You can only gain a treasure costing at most 3 more than the trashed card.').
		^self.
	].
	
	deck trashCard: trashCard from: #hand.
	deck gainCard: gainCard to: #hand.
	
	board currentDeck cardResolved: true.! !

ActionCard subclass: #ResponseCard
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

ResponseCard subclass: #Moat
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Moat class methodsFor: 'accessing' stamp: 'JohnParker 7/18/2012 20:01'!
cardName
	^'Moat'! !

!Moat class methodsFor: 'accessing' stamp: 'JohnParker 7/18/2012 20:01'!
cost
	^2! !

!Moat methodsFor: 'game' stamp: 'JohnParker 7/18/2012 21:19'!
playCard: board
	|deck|
	deck := board currentDeck.
	2 timesRepeat: [deck drawCardTo: #hand].
	! !

!Moat methodsFor: 'game' stamp: 'JohnParker 7/18/2012 20:00'!
revealResponse: board deck: deck
	deck attack: nil.! !

!ResponseCard methodsFor: 'game' stamp: 'JohnParker 7/18/2012 19:57'!
revealResponse: board deck: deck! !

ActionCard subclass: #Smithy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Smithy class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 13:57'!
cardName
	^'Smithy'! !

!Smithy class methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 23:30'!
cost
	^4! !

!Smithy methodsFor: 'game' stamp: 'JohnParker 7/18/2012 21:19'!
playCard: board
	3 timesRepeat: [board currentDeck drawCardTo: #hand].! !

ActionCard subclass: #Village
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Village class methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 00:07'!
cardName
	^'Village'! !

!Village class methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 23:30'!
cost
	^3! !

!Village methodsFor: 'game' stamp: 'JohnParker 7/18/2012 21:19'!
playCard: board
	|deck|
	deck := board currentDeck.
	deck drawCardTo: #hand.
	deck addActions: 2.
	! !

!Card class methodsFor: 'game' stamp: 'JohnParker 7/18/2012 12:08'!
buy: board
	^board currentDeck gainCard: self to: #discard.! !

!Card class methodsFor: 'accessing' stamp: 'JohnParker 7/3/2012 20:37'!
cardName
	^'Card' ! !

!Card class methodsFor: 'accessing' stamp: 'JohnParker 7/3/2012 20:37'!
cost
	^0.! !

!Card class methodsFor: 'instance creation' stamp: 'JohnParker 7/3/2012 20:03'!
new
	self error: 'Cards can only be created by the Board'! !

!Card methodsFor: 'accessing' stamp: 'JohnParker 7/3/2012 20:37'!
cardName
	^self class cardName.! !

!Card methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/10/2012 22:11'!
color
	^self subclassResponsibility! !

!Card methodsFor: 'accessing' stamp: 'JohnParker 7/3/2012 20:37'!
cost
	^self class cost.! !

!Card methodsFor: 'game' stamp: 'JohnParker 7/18/2012 11:47'!
play: board
	|ret|
	(board currentDeck moveCard:  self class from: #hand to: #play) ifFalse: [self Error: 'You do not have a ' , self cardName , ' in your hand!!'].
	
	[ret := self playCard: board] on: Error do: [:e | 
		board currentDeck moveCard: self class from: #play to: #hand.
		e pass.
	].
	^ret.! !

!Card methodsFor: 'private' stamp: 'JohnParker 7/18/2012 11:47'!
playCard: board
	self SubclassResponsibility ! !

Card subclass: #TreasureCard
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!TreasureCard commentStamp: 'JohnParker 7/4/2012 15:05' prior: 0!
A TreasureCard is an abstract class representing a Dominion treasure card.  It adds the value message which returns the amount of treasure the card is worth.!

TreasureCard subclass: #Copper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Copper class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:59'!
cardName 
	^'Copper'! !

!Copper class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 01:45'!
cost
	^0! !

!Copper methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:22'!
value
	^1! !

TreasureCard subclass: #Gold
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Gold class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:59'!
cardName 
	^'Gold'! !

!Gold class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:59'!
cost
	^6! !

!Gold methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:59'!
value
	^3! !

TreasureCard subclass: #Silver
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Silver class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:59'!
cardName 
	^'Silver'! !

!Silver class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:59'!
cost
	^3! !

!Silver methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:58'!
value
	^2! !

!TreasureCard class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 01:40'!
cardName
	^'Treasure'! !

!TreasureCard methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/10/2012 22:11'!
color
	^Color yellow! !

!TreasureCard methodsFor: 'private' stamp: 'JohnParker 7/18/2012 11:58'!
playCard: board
	board currentDeck addMoney: self value.! !

!TreasureCard methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:07'!
value
	self SubclassResponsibility! !

Card subclass: #VictoryCard
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!VictoryCard commentStamp: 'JohnParker 7/4/2012 15:06' prior: 0!
A VictoryCard is an abstract class representing a Dominion victory card.  It adds the value message which returns the number of victory points the card is worth. 
It also overrides the playCard message to generate an error since victory cards can not be played.!

VictoryCard subclass: #Curse
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Curse class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 16:06'!
cardName 
	^'Curse'! !

!Curse class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 16:06'!
cost
	^0! !

!Curse methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/10/2012 22:12'!
color
	^Color purple! !

!Curse methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 16:06'!
value
	^-1! !

VictoryCard subclass: #Duchy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Duchy class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 16:00'!
cardName 
	^'Duchy'! !

!Duchy class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 16:00'!
cost
	^5! !

!Duchy methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 16:00'!
value
	^3! !

VictoryCard subclass: #Estate
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Estate class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 01:49'!
cardName 
	^'Estate'! !

!Estate class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 01:49'!
cost
	^2! !

!Estate methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:23'!
value
	^1! !

VictoryCard subclass: #Province
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Province class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 16:00'!
cardName 
	^'Provinve'! !

!Province class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 16:00'!
cost
	^8! !

!Province methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 16:00'!
value
	^6! !

!VictoryCard class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 01:47'!
cardName 
	^'Victory'! !

!VictoryCard methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/10/2012 22:12'!
color
	^Color green! !

!VictoryCard methodsFor: 'private' stamp: 'JohnParker 7/18/2012 11:58'!
playCard: board
	self Error: 'Victory cards can not be played'! !

!VictoryCard methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:07'!
value
	self SubclassResponsibility ! !

Object subclass: #Command
	instanceVariableNames: 'deckNumber'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!Command commentStamp: 'JohnParker 7/4/2012 14:53' prior: 0!
A Command is an abstract class implementing the command object pattern.  Its reciever is specified by deckNumber.

Instance Variables
	deckNumber:		Number

deckNumber
	- The deck number of the reciever.
!

Command subclass: #CardCommand
	instanceVariableNames: 'cardClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!CardCommand commentStamp: 'JohnParker 7/4/2012 14:52' prior: 0!
A CardCommand is an abstract class representing a Command that accepts a card class as a parameter

Instance Variables
	cardClass:		Card class

cardClass
	- passed to the reciever as an argument
!

CardCommand subclass: #ActionCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!ActionCommand commentStamp: 'JohnParker 7/4/2012 14:50' prior: 0!
An ActionCommand is a command object that sends the playAction: cardClass message to the deck specified by deckNumber!

!ActionCommand methodsFor: 'run' stamp: 'JohnParker 7/18/2012 12:02'!
do: board
	[(board deck: deckNumber) playAction: cardClass] on: Error do: [:e | 
		(board deck: deckNumber) triggerEvent: #command:error: withArguments: {self. e messageText}.
	].
	! !

CardCommand subclass: #BuyCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!BuyCommand commentStamp: 'JohnParker 7/4/2012 14:51' prior: 0!
A BuyCommand is a command object that sends the buy: cardClass message to the deck specified by deckNumber!

!BuyCommand methodsFor: 'run' stamp: 'JohnParker 7/18/2012 12:02'!
do: board
	[(board deck: deckNumber) buy: cardClass] on: Error do: [:e | 
		(board deck: deckNumber) triggerEvent: #command:error: withArguments: {self. e messageText}.
	].
	! !

!CardCommand class methodsFor: 'instance creation' stamp: 'JohnParker 7/4/2012 14:20'!
deck: aNumber card: aCardClass
	^self new deck: aNumber card: aCardClass.! !

!CardCommand methodsFor: 'initialize-release' stamp: 'JohnParker 7/4/2012 14:20'!
deck: aNumber card: aCardClass
	deckNumber := aNumber.
	cardClass := aCardClass.
	! !

CardCommand subclass: #RevealResponseCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!RevealResponseCommand methodsFor: 'run' stamp: 'JohnParker 7/18/2012 20:06'!
do: board
	[(board deck: deckNumber) revealResponse: cardClass] on: Error do: [:e | 
		(board deck: deckNumber) triggerEvent: #command:error: withArguments: {self. e messageText}.
	].! !

CardCommand subclass: #TreasureCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!TreasureCommand commentStamp: 'JohnParker 7/4/2012 14:51' prior: 0!
A TreasureCommand is a command object that sends the playTreasure: cardClass message to the deck specified by deckNumber!

!TreasureCommand methodsFor: 'run' stamp: 'JohnParker 7/18/2012 12:10'!
do: board
	[(board deck: deckNumber) playTreasure: cardClass] on: Error do: [:e | 
		(board deck: deckNumber) triggerEvent: #command:error: withArguments: {self. e messageText}.
	].
	
	! !

!Command class methodsFor: 'instance creation' stamp: 'JohnParker 7/4/2012 14:20'!
deck: aNumber
	^self new deck: aNumber.! !

!Command methodsFor: 'initialize-release' stamp: 'JohnParker 7/4/2012 14:21'!
deck: aNumber
	deckNumber := aNumber.
	! !

!Command methodsFor: 'run' stamp: 'JohnParker 7/18/2012 12:01'!
do: board
	self SubclassResponsibility.! !

Command subclass: #EndTurnCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!EndTurnCommand commentStamp: 'JohnParker 7/4/2012 15:04' prior: 0!
An EndTurnCommand is a command object that sends the endTurn message to the deck specified by deckNumber
!

!EndTurnCommand methodsFor: 'run' stamp: 'JohnParker 7/18/2012 12:03'!
do: board
	[(board deck: deckNumber) cleanup] on: Error do: [:e | 
		(board deck: deckNumber) triggerEvent: #command:error: withArguments: {self. e messageText}.
	].! !

Command subclass: #ResolveActionCommand
	instanceVariableNames: 'args'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!ResolveActionCommand commentStamp: 'JohnParker 7/10/2012 19:57' prior: 0!
A ResolveActionCommand is a command sent by a player to inform their deck of the way in which they would like to resolve the action in play.  Should be sent in response to a #resolveAction: event from their Deck.

Instance Variables
	argVector:		Array

argVector
	- The list of arguments to be passed.  May vary from card to card.
!

!ResolveActionCommand class methodsFor: 'instance creation' stamp: 'JohnParker 7/6/2012 00:19'!
deck: aNumber argVector: anArgVector
	^self new deck: aNumber argVector: anArgVector! !

!ResolveActionCommand class methodsFor: 'instance creation' stamp: 'JohnParker 7/18/2012 21:32'!
deck: aNumber args: aDict
	^self new deck: aNumber args: aDict! !

!ResolveActionCommand methodsFor: 'initialize-release' stamp: 'JohnParker 7/18/2012 21:32'!
deck: aNumber args: aDict
	deckNumber := aNumber.
	args := aDict.! !

!ResolveActionCommand methodsFor: 'run' stamp: 'JohnParker 7/18/2012 21:40'!
do: board
	[(board deck: deckNumber) resolveAction: args] on: Error do: [:e | 
		(board deck: deckNumber) triggerEvent: #command:error: withArguments: {self. e messageText}.
	].
	! !

Command subclass: #ResolveAttackCommand
	instanceVariableNames: 'args'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!ResolveAttackCommand class methodsFor: 'instance creation' stamp: 'JohnParker 7/18/2012 12:53'!
deck: aNumber args: aDict
	^self new deck: aNumber args: aDict! !

!ResolveAttackCommand methodsFor: 'initialize-release' stamp: 'JohnParker 7/18/2012 12:54'!
deck: aNumber args: aDict
	deckNumber := aNumber.
	args := aDict.! !

!ResolveAttackCommand methodsFor: 'run' stamp: 'JohnParker 7/18/2012 12:54'!
do: board
	[(board deck: deckNumber) resolveAttack: args] on: Error do: [:e | 
		(board deck: deckNumber) triggerEvent: #command:error: withArguments: {self. e messageText}.
	].
	! !

Object subclass: #Deck
	instanceVariableNames: 'deckNumber board areas actions buys phase money resolved attackedBy'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!Deck commentStamp: 'JohnParker 7/17/2012 22:01' prior: 0!
A Deck represents all the cards in a Dominion Player's possesion and that player's current turn state.  A card can either be in the draw, discard, hand, play, peek, or reveal areas.  It can only be in one.  A card can only be removed from the deck by being trashed, which is to say it is moved to the trash area of the board.

Instance Variables
	areas:		Dictionary: Symbol -> CardArea
	actions: 	Number
	buys: 		Number
	money: 	Number
	phase: 		Symbol
	resolving:	Boolean
	
areas
	- This is a dictionary containing all of the places a card in a players posession may be.  Only the following symbols are allowed as keys:		
	#discard
		- The players discard pile.

	#draw
		- The players draw pile.  When this pile is empty the discard pile is shuffled and becomes the new draw pile.

	#hand
		- The player's current hand.  Drawn as a set of 5 cards during the cleanup phase.
	
	#play
		- The play area.  Played cards are placed in this area until the cleanup phase.
	
	#peek
		- A list of cards currently being "peeked" at by the player.  Used by cards that require a player to look at cards from their deck and make a decision.
	
	#reveal
		- A list of cards currently being "revealed" by the player.  Used by cards that require a player to reveal cards from their deck.


actions
	- The number of actions remaining to be played.

buys
	- The number of buys a player can still make.

money
	- The amount of money the player has this turn due to playing treasure or action cards that give extra money.

phase
	- The phase of the current Turn.  Can be one of:
		#Action 	The player can play actions, treasure, buy cards, or end their turn during this phase
		#Treasure 	The player can play treasure, buy cards, or end their turn during this phase
		#Buy 		The player can buy cards or end their turn during this phase
		#None 		It is not currently the players turn.  Any action will generate an error.

resolving
	- True if the deck is waiting for the player to resolve the last card played.

!

!Deck class methodsFor: 'instance creation' stamp: 'JohnParker 7/18/2012 11:43'!
number: aNumber board: aBoard
	^self new number: aNumber board: aBoard.! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 00:11'!
actions
	^actions! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 00:06'!
addActions: aNumber
	actions := actions + aNumber.! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 00:07'!
addBuys: aNumber
	buys := buys + aNumber! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 01:43'!
addMoney: aNumber
	money := money + aNumber! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/17/2012 20:48'!
allCards
	|all|
	all := OrderedCollection new.
	areas do: [:each | all addAll: each].
	^all.! !

!Deck methodsFor: 'attacks' stamp: 'JohnParker 7/18/2012 13:20'!
attack: aCard
	attackedBy := aCard.
	self triggerEvent: #attack: withArguments: {aCard class}. ! !

!Deck methodsFor: 'game' stamp: 'JohnParker 7/18/2012 21:11'!
buy: aCardClass
	(board currentDeck = self) ifFalse: [self error: 'It is not your turn!!'].
	(aCardClass cost > money) ifTrue: [self error: 'You do not have enough money to purchase a ' , aCardClass cardName].
	(buys = 0) ifTrue: [self cleanup. self error: 'You are out of buys.'].
	resolved ifFalse: [self error: 'You can not buy a card while resolving another card.'].
	
	(aCardClass buy: board) ifTrue: [
		buys := buys - 1.
		(buys = 0) ifTrue: [self cleanup].
	] ifFalse: [self error: 'Could not buy a ' , aCardClass cardName]
	
	
	! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 00:11'!
buys
	^buys! !

!Deck methodsFor: 'private' stamp: 'JohnParker 7/17/2012 20:54'!
card: aCardClass from: area1 to: area2
	|card|
	card := (areas at: area1) detect: [:each | each class = aCardClass] ifNone: [^false].
	(areas at: area1) remove: card ifAbsent: [^false].
	(areas at: area2) addLast: card.
	^true.! !

!Deck methodsFor: 'game' stamp: 'JohnParker 7/18/2012 21:11'!
cardResolved: val
	resolved := val! !

!Deck methodsFor: 'game' stamp: 'JohnParker 7/18/2012 21:11'!
cleanup
	(board currentDeck = self) ifFalse: [self error: 'It is not your turn!!'].
	resolved ifFalse: [self error: 'You can not end your turn while resolving a card.'].
	self default.
	board nextDeck.
	! !

!Deck methodsFor: 'private' stamp: 'JohnParker 7/18/2012 21:14'!
default
	actions := 1.
	buys := 1.
	money := 0.
	phase := #None.
	resolved := true.
	
	(areas at: #discard) addAllLast: self play.
	self play removeAll.
	
	(areas at: #discard) addAllLast: self hand.
	self hand removeAll.
	
	1 to: 5 do: [:a | self drawCardTo: #hand].! !

!Deck methodsFor: 'private' stamp: 'JohnParker 7/17/2012 20:56'!
drawCard
	| card draw discard|
	"Reshuffle the discard into the draw if the draw is empty"
	draw := areas at: #draw.
	discard := areas at: #discard.
	
	(draw size = 0) ifTrue: [
		draw addAllLast: discard. 
		discard removeAll. 
		draw shuffle.
		self triggerEvent: #deckShuffled: withArguments: {deckNumber}.
	].
	"If the draw is still empty, we have no cards to draw so do nothing"
	(draw size = 0) ifFalse: [
		card := draw last.
		draw removeLast.
	].
	self  triggerEvent: #cardDrawn: withArguments: {card class}.
	^card.
	! !

!Deck methodsFor: 'card-moving' stamp: 'JohnParker 7/17/2012 21:35'!
drawCardTo: anArea
	| card |
	card := self drawCard.
	card isNil ifFalse: [
		(areas at: anArea) add: card.
	].

	self triggerMoveEvent: card class from: #draw to: anArea.
	^card.
	! !

!Deck methodsFor: 'card-moving' stamp: 'JohnParker 7/18/2012 11:45'!
gainCard: aCardClass to: anArea
	|card|
	card := board takeCard: aCardClass deck: deckNumber.
	card isNil ifFalse: [
		(areas at: anArea) addLast: card.
		self triggerEvent: #deck:cardMovedGlobal:from:to: withArguments: {deckNumber. aCardClass. #board. anArea}.
		^true.
	].
	^false.
! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/17/2012 20:48'!
hand
	^areas at: #hand! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/17/2012 20:48'!
handContains: aCardClass
	^(self hand detect: [:each | each class = aCardClass] ifNone: [nil]) isNil not.
	! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/17/2012 20:49'!
handContainsAll: aCardClassVector
	|set|
	set := aCardClassVector asSet.
	^set = (set select: [:testCardClass | 
		(self hand count: [:handCard | handCard class = testCardClass]) >=  
			(aCardClassVector count: [:each | each class = testCardClass])
	]).
	! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 13:22'!
money
	^money! !

!Deck methodsFor: 'card-moving' stamp: 'JohnParker 7/17/2012 21:57'!
moveCard: aCardClass from: area1 to: area2
	|card|
	
	((areas at: area1) visibility = #hidden) ifTrue: [^false].
	card := (areas at: area1) detect: [:each | each class = aCardClass] ifNone: [^false].
	(areas at: area1) remove: card ifAbsent: [^false].
	(areas at: area2) addLast: card.
	
	self triggerMoveEvent: aCardClass from: area1 to: area2.
	^true.! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 20:29'!
number
	^deckNumber! !

!Deck methodsFor: 'initialize-release' stamp: 'JohnParker 7/18/2012 11:42'!
number: aNumber board: aBoard
	deckNumber := aNumber.
	board := aBoard.
	areas := Dictionary new.
	areas at: #discard put: (CardArea visibility: #global).
	areas at: #draw put: (CardArea visibility: #hidden).
	areas at: #hand put: (CardArea visibility: #player).
	areas at: #play put: (CardArea visibility: #global).
	areas at: #peek put: (CardArea visibility: #player).
	areas at: #reveal put: (CardArea visibility: #global).
	
	1 to: 7 do: [:a | self gainCard: Copper to: #discard].
	1 to: 3 do: [:a | self gainCard: Estate to: #discard].
	self default.
	
	! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 12:45'!
phase
	^phase! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/17/2012 20:49'!
play
	^areas at: #play! !

!Deck methodsFor: 'game' stamp: 'JohnParker 7/18/2012 21:20'!
playAction: aCardClass
	| card |
	(board currentDeck = self) ifFalse: [self error: 'It is not your turn!!'].
	(aCardClass inheritsFrom: ActionCard) ifFalse: [self error: aCardClass cardName , ' is not an action card.'].
	(phase = #Treasure) ifTrue: [self error: 'You can not play an action during the treasure phase.'].
	(phase = #Buy) ifTrue: [self error: 'You can not play an action during the buy phase.'].
	resolved ifFalse: [self error: 'You can not play an action while resolving another card.'].
	
	card := self hand detect: [:each | each class = aCardClass] ifNone: [self error: 'You do not have a ' , aCardClass cardName , ' in your hand.'].
	
	actions := actions - 1.
	
	card play: board.
	
	(actions = 0) ifTrue: [phase := #Treasure].! !

!Deck methodsFor: 'game' stamp: 'JohnParker 7/18/2012 21:12'!
playTreasure: aCardClass
	| card |
	(board currentDeck = self) ifFalse: [self error: 'It is not your turn!!'].
	(aCardClass inheritsFrom: TreasureCard) ifFalse: [self error: aCardClass cardName , ' is not a treasure card.'].
	(phase = #Buy) ifTrue: [self error: 'You can not play a treasure during the buy phase.'].
	resolved ifFalse: [self error: 'You can not play a treasure while resolving another card.'].
	
	card := self hand detect: [:each | each class = aCardClass] ifNone: [self error: 'You do not have a ' , aCardClass cardName , ' in your hand.'].
	
	card play: board.
	phase := #Treasure.
	
	self triggerEvent: #cardPlayed withArguments: {deckNumber. aCardClass}.! !

!Deck methodsFor: 'game' stamp: 'JohnParker 7/18/2012 21:20'!
resolveAction: argVector
	(board currentDeck = self) ifFalse: [self error: 'It is not your turn!!'].
	resolved ifTrue: [self error: 'There is no action to resolve right now.'].
	
	self play last resolve: argVector board: board.
	! !

!Deck methodsFor: 'game' stamp: 'JohnParker 7/18/2012 13:02'!
resolveAttack: args
	(board currentDeck = self) ifTrue: [self error: 'You can not be attacked on your turn.'].
	attackedBy ifNil: [^self].
	
	(attackedBy resolveAttack: args board: board deck: self) ifTrue: [attackedBy := nil].
	! !

!Deck methodsFor: 'game' stamp: 'JohnParker 7/18/2012 20:16'!
revealResponse: aCardClass
	| card |
	(board currentDeck = self) ifTrue: [self error: 'You can not be attacked on your turn.'].
	attackedBy ifNil: [^self].
	
	card := self hand detect: [:each | each class = aCardClass] ifNone: [self error: 'You do not have a ' , aCardClass cardName , ' in your hand.'].
	
	self moveCard: card class from: #hand to: #reveal.
	self moveCard: card class from: #reveal to: #hand.
	card revealResponse: board deck: self.
	
	
	! !

!Deck methodsFor: 'game' stamp: 'JohnParker 7/13/2012 13:25'!
startTurn
	phase := #Action.
	self triggerEvent: #playerTurn withArguments: {deckNumber} .! !

!Deck methodsFor: 'card-moving' stamp: 'JohnParker 7/18/2012 11:45'!
trashCard: aCardClass from: anArea
	|card|
	card := (areas at: anArea) detect: [:each | each class = aCardClass] ifNone: [nil].
	card isNil ifFalse: [
		(areas at: anArea) remove: card.
		board trashCard: card deck: aCardClass.
		self triggerEvent: #deck:cardMovedGlobal:from:to: withArguments: {deckNumber. aCardClass. anArea. #trash}.
		^true.
	].
	^false.
! !

!Deck methodsFor: 'private' stamp: 'JohnParker 7/17/2012 22:10'!
triggerMoveEvent: aCardClass from: area1 to: area2
	(((areas at: area1) visibility = #global) or: ((areas at: area2) visibility = #global)) ifTrue: [
		self triggerEvent: #deck:cardMovedGlobal:from:to: withArguments: {deckNumber. aCardClass. area1. area2}.
	] ifFalse: [
		self triggerEvent: #deck:cardMovedPlayer:from:to: withArguments: {deckNumber. aCardClass. area1. area2}.
	].! !

!Deck methodsFor: 'game' stamp: 'JohnParker 7/18/2012 11:45'!
victoryPoints
	(board gameOver) ifFalse: [self error: 'No peeking!! The game is not over yet!!'].
	^self allCards inject: 0 into: [:points :card |
		(card class inheritsFrom: VictoryCard) ifTrue: [points + card value] ifFalse: [points]
	]! !

!Deck methodsFor: 'testing' stamp: 'JohnParker 7/17/2012 20:58'!
z_addCardToHand: aCardClass
	self hand add: aCardClass basicNew! !

!Deck methodsFor: 'testing' stamp: 'JohnParker 7/17/2012 20:59'!
z_discard
	^(areas at: #discard)! !

!Deck methodsFor: 'testing' stamp: 'JohnParker 7/17/2012 20:59'!
z_draw
	^(areas at: #draw)! !

!Deck methodsFor: 'testing' stamp: 'JohnParker 7/18/2012 21:12'!
z_resolving 
	^resolved not ! !

Object subclass: #TestPlayer
	instanceVariableNames: 'board deckNumber errorCommand errorString'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Test'!

!TestPlayer commentStamp: 'JohnParker 7/10/2012 19:56' prior: 0!
A Player is an abstract class that represents a player.  In this design, a player controls the corresponding deck, sending commands to it in order to play and respond to cards.  A Player also listens for events from Board and Deck which inform it of changes to the game state.  The player may also display the current state of the board if it is a human player.

Instance Variables
	deckNumber:		Number

deckNumber
	- The number of the deck this player controls.
!

!TestPlayer methodsFor: 'events' stamp: 'JohnParker 7/18/2012 20:23'!
attack: aCardClass
	|card |
	
	card := (board deck: deckNumber) hand detect: [:each | each class inheritsFrom: ResponseCard] ifNone: [nil].
	card isNil ifFalse: [board execute: (RevealResponseCommand deck: deckNumber card: card class)].
	
	(aCardClass = Bureaucrat) ifTrue: [
		|command args|
		card := (board deck: deckNumber) hand detect: [:each | each class inheritsFrom: VictoryCard] ifNone: [command := ResolveAttackCommand deck: deckNumber args: Dictionary new. nil].
		
		(card isNil) ifFalse: [
			args := Dictionary new.
			args at: #bureaucratCard put: card class.
			command := ResolveAttackCommand deck: deckNumber args: args.
		].
		
		board execute: command.
	].
	
	(aCardClass = Witch) ifTrue: [
		board execute: (ResolveAttackCommand deck: deckNumber args: Dictionary new)
	]! !

!TestPlayer methodsFor: 'setup' stamp: 'JohnParker 7/18/2012 12:11'!
board: aBoard deck: aNumber
	board := aBoard.
	deckNumber := aNumber.! !

!TestPlayer methodsFor: 'accessing' stamp: 'JohnParker 7/17/2012 20:25'!
clearError
	errorCommand := nil.
	errorString := nil.! !

!TestPlayer methodsFor: 'events' stamp: 'JohnParker 7/17/2012 20:25'!
command: aCommand error: aString
	errorCommand := aCommand.
	errorString := aString.! !

!TestPlayer methodsFor: 'events' stamp: 'JohnParker 7/18/2012 12:13'!
doesNotUnderstand: aMessage
	(board events includes: aMessage selector) ifFalse: [super doesNotUnderstand: aMessage].
	! !

!TestPlayer methodsFor: 'accessing' stamp: 'JohnParker 7/17/2012 20:24'!
errorCommand
	^errorCommand! !

!TestPlayer methodsFor: 'accessing' stamp: 'JohnParker 7/17/2012 20:24'!
errorString
	^errorString! !

!TestPlayer methodsFor: 'events' stamp: 'JohnParker 7/18/2012 21:41'!
resolveAction: aCardClass
	|dict|
	dict := Dictionary new.
	
	(aCardClass = Cellar) ifTrue: [
		| victoryCards |
		victoryCards := (board deck: deckNumber) hand select: [:each | each class inheritsFrom: VictoryCard] thenCollect: [:each | each class].
		
		dict at: #discards put: victoryCards asArray.
		board execute: (ResolveActionCommand deck: deckNumber args: dict).
	].
	
	(aCardClass = Mine) ifTrue: [
		| treasureCards trashCard gainCard |
		treasureCards := (board deck: deckNumber) hand select: [:each | each class inheritsFrom: TreasureCard] thenCollect: [:each | each class].
		trashCard := treasureCards detectMin: [:each | each cost].
		gainCard := {Silver. Gold} detect: [:each | each cost = (trashCard cost + 3)].
		
		dict at: #trashCard put: trashCard.
		dict at: #gainCard put: gainCard.
		board execute: (ResolveActionCommand deck: deckNumber args: dict).
	].! !

!TestPlayer methodsFor: 'events' stamp: 'JohnParker 7/18/2012 21:47'!
resolveAction: aCardClass deck: aNumber
	|dict|
	dict := Dictionary new.
	
	(aCardClass = Spy) ifTrue: [
		|discard|
		discard := (board deck: aNumber) reveal last class inheritsFrom: TreasureCard.
		(deckNumber = aNumber) ifTrue: [discard := discard not].
		
		dict at: #deck put: #aNumber.
		dict at: #discard put: discard.
		board execute: (ResolveActionCommand deck: deckNumber args: dict).
	].! !
